/************************************************************
 *
 * $Id: main.c,v 1.7 2000/05/03 19:25:05 htran Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 ************************************************************/

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "arguments.h"
#include "uTypes.h"
#include "listeners.h"

/************************************************************
 *
 * Global mutexes
 *
 *
 ************************************************************/

pthread_mutex_t argumentsLock;
pthread_mutex_t playingLock;

/************************************************************
 *
 * Auxilliary functions within this file
 *
 ************************************************************/
void 
usage(void);

int 
get_arguments(args_t *arguments, const char *argv[]);

int 
parse_sdp_file(args_t *arguments);

int 
show_arguments(const args_t *a);

/************************************************************
 *
 * Main
 *
 ************************************************************/

int 
main(int argc, char *argv[]){
  args_t args;
  pthread_t videoThread;
  pthread_t audioThread;

  
  pthread_mutex_init(&argumentsLock, NULL);
  
  pthread_mutex_lock(&argumentsLock);
  if(argc==1)
	 usage();
  if(get_arguments(&args,(const char **)argv) < 0)
	 return -1;
  else
	 show_arguments(&args);
  
  pthread_mutex_unlock(&argumentsLock);
  
  if( args.playVideo==false && args.playAudio==false )
	 usage();
  
  //while(true) 
  // {
  //	i++;
  pthread_create(&videoThread,NULL,(void *)rtp_video_listener,&args);
  pthread_create(&audioThread,NULL,(void *)rtp_audio_listener,&args);
  
  
  pthread_join(videoThread,NULL);
  fprintf(stderr,"\nVideo is done\n");
  
  //  pthread_cancel(audioThread);
  pthread_join(audioThread,NULL);
  fprintf(stderr,"audio is done\n");
  // }
  
  return (1);
}

/************************************************************
 *
 * Show the arguments structure
 *
 ************************************************************/
int 
show_arguments(const args_t *a){
  fprintf(stderr,"videoAddress set to: %s %d\n",
			 (a->videoAddress!=NULL) ? a->videoAddress:"NULL",a->videoPort);
  fprintf(stderr,"audioAddress set to: %s %d\n",
			 (a->audioAddress!=NULL) ? a->audioAddress:"NULL", a->audioPort);
  fprintf(stderr,"sdpFile set to: %s\n",
			 (a->sdpFile!=NULL) ? a->sdpFile : "NULL");
  fprintf(stderr,"Video is %s\n", (a->playVideo)?"ON":"OFF");
  fprintf(stderr,"Audio is %s\n", (a->playAudio)?"ON":"OFF");
  fprintf(stderr,"Zoom is %s\n", (a->zoom)?"ON":"OFF");
  return 1; 
}

/************************************************************
 *
 * Parse a specified sdp file and fill the arguments structure
 *
 ************************************************************/

int 
parse_sdp_file(args_t *a){
  FILE *fp;
  char *dataBuffer=NULL;
  char *dup=NULL;
  char *tmp=NULL;
  int sizeOfFile=0;
  char *tmpVP=NULL;
  char *tmpVA=NULL;
  char *tmpAP=NULL;
  char *tmpAA=NULL;
  
  /* 
	  If we get "-f" anything other than ZOOM
	  passed on the command line is assumed to be junk, so
	  reset values, then parse sdp
  */
  memset(a->videoAddress,0,16);
  memset(a->audioAddress,0,16);
  a->videoPort=-1;
  a->audioPort=-1;
  a->playVideo=false;
  a->playAudio=false;
  
  if( (fp=fopen(a->sdpFile,"r"))<0 ){
	 perror("Opening sdpFile");
	 return -1;
  }
  /* read in the file to dataBuffer */
  fseek(fp, 0, FROM_END);
  sizeOfFile=ftell(fp);
  dataBuffer=(char *)malloc(sizeOfFile+1);
  fseek(fp,0,FROM_START);
  fread(dataBuffer,sizeOfFile,1,fp);

  if( fclose(fp)<0 ){
	 fprintf(stderr,"Couldn't close sdp file %s",a->sdpFile);
  }
  
  
  
  /* dup and parse the dup */
  dup=(char *)strdup(dataBuffer);
  tmp=(char *)strtok(dup,"=\n");
  tmp=(char *)strtok(NULL,"=\n");
  while( tmp!=NULL ){
	 if( strcmp(tmp,"m")==0 ){	  // found media tag
		tmp=(char *)strtok(NULL,"=\n");
		if( (char *)strstr(tmp,"video")!=NULL ){	//found video tag
		  tmpVP=(char *)strdup(tmp);
		  tmpVA=(char *)strtok(NULL,"=\n");
		  tmpVA=(char *)strtok(NULL,"=\n");
		  a->playVideo=true;
		}
		else if ( (char *)strstr(tmp,"audio")!=NULL ){	//found audio tag
		  tmpAP=(char *)strdup(tmp);
		  tmpAA=(char *)strtok(NULL,"=\n");
		  tmpAA=(char *)strtok(NULL,"=\n");
		  a->playAudio=true;
		} 
	 }
	 
	 tmp=(char *)strtok(NULL,"=\n");
  }
  
  /* found all 4 pieces of information, pack them into the args struct */
  
  tmp=(char *)strtok(tmpVP," ");
  tmp=(char *)strtok(NULL," ");
  a->videoPort=atoi(tmp);
  
  tmp=(char *)strtok(tmpAP," ");
  tmp=(char *)strtok(NULL," ");
  a->audioPort=atoi(tmp);
  
  tmp=(char *)strtok(tmpVA," ");
  tmp=(char *)strtok(NULL," ");
  tmp=(char *)strtok(NULL," ");
  tmp=(char *)strtok(tmp,"/");
  strcpy(a->videoAddress,tmp);

  tmp=(char *)strtok(tmpAA," ");
  tmp=(char *)strtok(NULL," ");
  tmp=(char *)strtok(NULL," ");
  tmp=(char *)strtok(tmp,"/");
  strcpy(a->audioAddress,tmp);
  //done  
  return 1;
}

/************************************************************
 *
 * Get the command line arguments
 *
 ************************************************************/

int 
get_arguments(args_t *a, const char *argv[]){
  int i=0;
  /* Init the arguments */
  memset(a->videoAddress,0,16);
  memset(a->audioAddress,0,16);
  a->videoPort=-1;
  a->audioPort=-1;
  a->playVideo=false;
  a->playAudio=false;
  a->zoom=false;
  a->sdpFile=NULL;
  
  for(i=0;argv[i]!=NULL;i++){
	 if( strcmp(argv[i],"-z")==0 )
		a->zoom=true;
	 else if( strcmp(argv[i],"-a")==0 ){
		if( argv[i+1]==NULL || argv[i+2]==0 ){
		  fprintf(stderr,"Not all AUDIO options have been specified\n");
		  usage();
		}
		strcpy(a->audioAddress, argv[++i]);
		a->audioPort=atoi(argv[++i]);
		a->playAudio=true;
	 } 
	 else if( strcmp(argv[i],"-v")==0 ){
		if( argv[i+1]==NULL || argv[i+2]==0 ){
		  fprintf(stderr,"Not all VIDEO options have been specified\n");
		  usage();
		}
		strcpy(a->videoAddress, argv[++i]);
		a->videoPort=atoi(argv[++i]);
		a->playVideo=true;
	 } 
	 else if ( strcmp(argv[i],"--help")==0 || strcmp(argv[i],"-h")==0 ||
				  strcmp(argv[i],"-u")==0 ){
		
		usage();
	 } 
	 else if ( strcmp(argv[i],"-f")==0 ){
		if( argv[i+1]==NULL ){
		  fprintf(stderr,"Need to specify a filename with -f option\n");
		  usage();
		} 
		else{
		  a->sdpFile=(char *)malloc(strlen(argv[++i])+1);
		  strcpy(a->sdpFile,argv[i]);
		  parse_sdp_file(a);
		  for(i=0;argv[i]!=NULL;i++){
			 if( strcmp(argv[i],"-noaudio")==0 )
				a->playAudio=false;
			 if( strcmp(argv[i],"-novideo")==0 )
				a->playVideo=false;
			 if( strcmp(argv[i],"-z")==0 )
				a->zoom=true;
		  }
		  
		  return 1;		  
		}
	 }	 
  }
  
  return 1;
  
}

/************************************************************
 *
 * Prints out the usage information of the program, ansi codes within
 *
 ************************************************************/

void 
usage(void){
  fprintf(stderr,"\n\033[1mmim\033[0m (isn't mtv)\n");
  fprintf(stderr,"----------------- \n");
  fprintf(stderr,"Usage: mim [-options [option arguments] ]\n\n");
  fprintf(stderr,"\033[1mmim\033[0m is an application that allows linux/unix boxen with multicast support to view broadcasted programs. \033[1mmim\033[0m has no defaults set.\n\n");
  fprintf(stderr,"Options:\n");
  fprintf(stderr,"\t--help\t\t\tThis usage message.\n");
  fprintf(stderr,"\t-u\t\t\tThis usage message.\n");
  fprintf(stderr,"\t-h\t\t\tThis usage message.\n");
  fprintf(stderr,"\t-f filename\t\tSpecify a SDP file to read group/port\n\t\t\t\tinformation from.\n");
  fprintf(stderr,"\t-v group port\t\tSpecify the group and port for video.\n\t\t\t\tIf this option is used alone, audio will not be\n\t\t\t\tpresented.\n");
  fprintf(stderr,"\t-a group port\t\tSpecify the group and port for audio.\n\t\t\t\tIf this option is used alone, video will not be\n\t\t\t\tpresented.\n");
  fprintf(stderr,"\t-z\t\t\tZoom the video\n");
  fprintf(stderr,"\t-noaudio\t\tDon't play audio\n");
  fprintf(stderr,"\t-novideo\t\tDon't play video\n");
  fprintf(stderr,"\n\n");
  exit(-1);
}

  







