/**********************************************************************
 *
 * $Id: rtp.h,v 1.3 2000/05/03 19:25:05 htran Exp $
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 **********************************************************************/

								

#ifndef RTP_H
#define RTP_H					

#include "uTypes.h"

#define BUFFERSIZE 4096
#define RTP_SDES_MAX 255
#define RTP_SEQ_MOD (1<<16)


/* BEGIN RTP STRUCTURES */
/* these are defined in RFC1889 */
/* rtp header */
				
typedef struct {
#ifdef RTP_BIG_ENDIAN			  /* IF BIGENDIAN */
  uint16_t version:2;			  /* proto version */
  uint16_t p:1;					  /* padding */
  uint16_t x:1;					  /* header extension: should be 0 */
  uint16_t cc:4;					  /* CSRC count */
  uint16_t m:1;					  /* marker bit */
  uint16_t pt:7;					  /* payload type */
#endif
#ifdef RTP_LITTLE_ENDIAN		  /* IF LILENDIAN */
  uint16_t cc:4;					  /* csrc count */
  uint16_t x:1;					  /* header extension should be set to 0 */
  uint16_t p:1;					  /* padding */
  uint16_t version:2;			  /* rtp protocol version */
  uint16_t pt:7;					  /* payload type */
  uint16_t m:1;					  /* marker bit */
#endif 
  uint16_t  seq;					  /* sequence number */
  uint32_t  ts;					  /* timestamp */
  uint32_t  ssrc;					  /* sync source */
}rtpHeader_t;

typedef struct {
#if RTP_BIG_ENDIAN
  uint32_t MBZ:5;					  /* should be zero, unused */
  uint32_t T:1;					  /* 1 means mpeg2 ext is used,0 otherwise */
  unsigned short TR:10;			  /* temporal ref 0-1023 same for a 
											  picture */ 
  uint32_t AN:1;					  /* should be 0 for mpeg-1 */  
  uint32_t N:1;					  /* should be 0 for mpeg-1 */
  uint32_t S:1;					  /* sequence header present bit */
  uint32_t B:1;					  /* beginning of slice bit */
  uint32_t E:1;		
  uint32_t P:3;					  /* picture type: I,P,B,D */
  uint32_t FBV:1;		
  uint32_t BFC:3;		  
  uint32_t FFV:1;
  uint32_t FFC:3;
#elif RTP_LITTLE_ENDIAN  
  unsigned short TR:10;
  uint32_t T:1;
  uint32_t MBZ:5;
  uint32_t P:3;
  uint32_t E:1;
  uint32_t B:1;
  uint32_t S:1;
  uint32_t N:1;
  uint32_t AN:1;
  
  uint32_t FFC:3;
  uint32_t FFV:1;
  uint32_t BFC:3;
  uint32_t FBV:1;    
#endif
} rtpVideoHeader_t;


typedef struct {
  uint16_t  MBZ;			/* should be zero, unused */
  uint16_t  F_OFFSET;
}rtpAudioHeader_t;


/* rtp_video_packet */
typedef struct {  
  rtpHeader_t rtpHead;			
  /* x bit should be zero for mpeg video so we skip the rtp_hdr_ext junk */ 
  rtpVideoHeader_t videoHead;
  uchar_t data[BUFFERSIZE]; 
}rtpVideoPacket_t;

/* rtp_audio_packet*/
typedef struct {
  rtpHeader_t rtpHead;
  rtpAudioHeader_t audioHead;
  unsigned char data[BUFFERSIZE];  
}rtpAudioPacket_t;


/* END RTP STRUCTURES */

typedef enum {
  RTCP_SR    =200,
  RTCP_RR    =201,
  RTCP_SDES  =202,
  RTCP_BYE   =203,
  RTCP_APP   =204,
}rtcpType_t;

typedef enum {
  RTCP_SDES_END   = 0,
  RTCP_SDES_CNAME = 1,
  RTCP_SDES_NAME  = 2,
  RTCP_SDES_EMAIL = 3,
  RTCP_SDES_PHONE = 4,
  RTCP_SDES_LOC   = 5,
  RTCP_SDES_TOOL  = 6,
  RTCP_SDES_NOTE  = 7,
  RTCP_SDES_PRIV  = 8
}rtcpSdesType_t;

typedef struct {
#ifdef RTP_BIG_ENDIAN
  uchar_t version:2;				  /* protocol version */
  uchar_t p:1;						  /* padding flag */
  uchar_t count:5;				  /* varies by packet type */
#endif
#ifdef RTP_LITTLE_ENDIAN
  uchar_t count:5;				  /* varies by packet type */
  uchar_t p:1;						  /* padding flag */
  uchar_t version:2;				  /* protocol version */
#endif
  uchar_t pt;						  /* RTCP packet type */
  uint16_t length;				  /* pkt len in words, w/o 
											  this word */
} rtcpCommon_t;

#define RTCP_VALID_MASK (0xc000 | 0x2000 | 0xfe)
#define RTCP_VALID_VALUE ((RTP_VERSION << 14) | RTCP_SR)

typedef struct {
  uint32_t ssrc;					  /* data source being reported */
  uchar_t  fraction;				  /* fraction lost since last SR/RR */
  int      lost;					  /* cumul. no. pkts lost (signed!) */
  uint32_t last_seq;				  /* extended last seq. no. received */
  uint32_t jitter;				  /* interarrival jitter */
  uint32_t lsr;					  /* last SR packet from this source */
  uint32_t dlsr;					  /* delay since last SR packet */
} rtcpRR_t;


typedef struct {
  uchar_t type;					  /* type of item (rtcp_sdes_type_t) */
  uchar_t length;					  /* length of item (in octets) */
  char data[1];					  /* text, not null-terminated */
} rtcpSdesItem_t;


typedef struct {
  rtcpCommon_t common;			  /* common header */
  union {
	 /* sender report (SR) */
    struct {
      uint32_t ssrc;				  /* sender generating this report */
      uint32_t ntp_sec;			  /* ntp timestamp */
      uint32_t ntp_frac;
      uint32_t rtp_ts;			  /* RTP timestamp */
      uint32_t psent;				  /* packets sent */
      uint32_t osent;				  /* octets sent */ 
      rtcpRR_t rr[1];			  /* variable-length list */
    } sr;
	 
	 /* reception report (RR) */
    struct {
      uint32_t ssrc;				  /* receiver generating this report */
      rtcpRR_t rr[1];			  /* variable-length list */
    } rr;
    
	 /* source description (SDES) */
    struct rtcpSdes {
      uint32_t src;				  /* first SSRC/CSRC */
      rtcpSdesItem_t item[1];	  /* list of SDES items */
    } sdes;
    
	 /* BYE */
    struct {
      uint32_t src[1];	/* list of sources */
		/* can't express trailing text for reason */
    } bye;
  } r;
} rtcp_t;

typedef struct rtcpSdes rtcpSdes_t;
#endif






