/************************************************************
 *
 * $Id: rtp_audio_listener.c,v 1.5 2000/05/03 19:25:05 htran Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ************************************************************/

#include <netdb.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include "listeners.h"
#include "arguments.h"
#include "start_msock.h"



extern pthread_mutex_t playingLock;

static void 
close_opened_FILE(FILE *fp){
  while( fclose(fp)< 0 );
  pthread_mutex_unlock(&playingLock);
  
}

static void 
close_opened_fd(int *fd){
  while( close(*fd)< 0 );
}

static void 
fill_send_item(rtcpSendItem_t *sendBuffer, const aMediaSource_t *source);

void *
rtp_audio_listener(args_t *args){ 
  FILE *fpOut;
  int errorStatus=-1;
  int pthreadLastState;
  fd_set readFdSet, activeFdSet;  
  aMediaSource_t audioSource;
  extern pthread_mutex_t argumentsLock;
  
  
  rtcpSendItem_t sItem;
  struct sockaddr_in sendToAddress;
  socklen_t addressSize=sizeof(struct sockaddr_in);

  rtcp_t recvRTCP;
  
  rtpAudioPacket_t recvPacket;
  size_t rtpPacketSize=sizeof(recvPacket);
  int length=0;
  int sendSize=0;
  
  int i=0;
  bool doneSleeping=false;
  
  
  if(args==NULL)
	 pthread_exit(&errorStatus);
  /* lock argumentsLock, copy data, unlock */
  pthread_mutex_lock(&argumentsLock);
  strcpy(audioSource.address, args->audioAddress);
  audioSource.port=args->audioPort;
  audioSource.zoom=args->zoom;
  audioSource.enabled=args->playAudio;
  pthread_mutex_unlock(&argumentsLock);
  
  if( !audioSource.enabled )
	 pthread_exit(&errorStatus);
  else{
	 if( (audioSource.rtcpsock=
			start_msock(audioSource.address,audioSource.port+1,1,127,0))< 0 ){
		perror("rtcp_socks[audio]=start_msock()");
		exit(-1);
	 }
	 else{
		fprintf(stderr,"rtcpAudio started\n");
	 }
	 if( (audioSource.rtpsock=
			start_msock(audioSource.address,audioSource.port,1,127,0))< 0 ){
		perror("rtp_socks[audio]=start_msock()");
		exit(-1);
	 } 
	 else{
		fprintf(stderr,"rtpAudio started\n");
	 }
  }
  
  /* if we get here we know that audio will be played */
  pthread_mutex_trylock(&playingLock);
  
  
  if( (fpOut=popen("mpg123 -m -", "w"))==NULL ){
	 fprintf(stderr,"%d:: ",(int)pthread_self());
	 perror("popen()");
	 pthread_exit(&errorStatus);
  }
  
  fill_send_item(&sItem,&audioSource);
  fprintf(stderr,"%s\n", sItem.sdes.item.data);
  
  /* set up destination address */
  sendToAddress.sin_family=AF_INET;
  sendToAddress.sin_addr.s_addr=inet_addr(audioSource.address);
  sendToAddress.sin_port=htons(audioSource.port+1);  

  /* set to asynchronous cancel */
  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &pthreadLastState);
  
  /* set up file sets */
  FD_ZERO(&activeFdSet);
  FD_SET(audioSource.rtpsock,&activeFdSet);
  FD_SET(audioSource.rtcpsock,&activeFdSet);
  
  
  /* establish cleanup handlers for the open socks and FILE */
  pthread_cleanup_push((void *)close_opened_FILE,(void *)fpOut);
  pthread_cleanup_push((void *)close_opened_fd,(void *)&audioSource.rtpsock);
  pthread_cleanup_push((void *)close_opened_fd,
							  (void *)&audioSource.rtcpsock);
  pthread_cleanup_push((void *)stop_msock,(void *)&audioSource);   
  /* enter main loop and don't leave */
  
  while(1){	
	 readFdSet=activeFdSet;
	 memset(&recvPacket,0,rtpPacketSize);
	 memset(recvPacket.data,0,BUFFERSIZE);   
	 sendSize=0;
	 length=0;
	 if (select (FD_SETSIZE, &readFdSet, NULL, NULL, NULL) < 0){
		perror ("select");
		exit (-1);
	 }
	 for(i=0;i<FD_SETSIZE;++i){
		if( FD_ISSET(i,&readFdSet) ){
		  if( i==audioSource.rtpsock ){   // RTP Ready
			 if(!doneSleeping)
				sleep(4);
			 doneSleeping=true; 
			 if( (length=recvfrom(audioSource.rtpsock,
										 (rtpAudioPacket_t *)&recvPacket,rtpPacketSize,0,
										 NULL,NULL)) < 0 ){
				perror("rtp:recvfrom()");
			 }
			 sendSize=length-sizeof(rtpHeader_t)-sizeof(rtpAudioHeader_t);
			 fwrite(recvPacket.data,sendSize,1,fpOut);
		  }
		  else if( i==audioSource.rtcpsock ){   //RTCP Ready
			 
			 memset(&recvRTCP,0,sizeof(recvRTCP));
			 if( (length=recvfrom(audioSource.rtcpsock,(rtcp_t *)&recvRTCP,
										 sizeof(recvRTCP),0,
										 NULL,NULL)) < 0 ){
				perror("rtcp:recvfrom()");
				pthread_exit(&length);
			 }
			 
			 if(recvRTCP.common.pt==200){
				if( sendto(audioSource.rtcpsock,(rtcp_t *)&sItem,
							  sizeof(rtcpSendItem_t),0,
							  (struct sockaddr *)&sendToAddress,addressSize)<0 ){
				  fprintf(stderr,"%d:: ", (int)pthread_self());
				  perror("rtcp:sendto()");
				}
			 }
		  }
		}
	 }
  }
  
  

  
  
  /* should never get to here.
	  the thread should just loop until it gets a cancel signal from
	  parent.
	  Code below here exists for conformity with standards.
  */
  return NULL;

  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 
}
static void 
fill_send_item(rtcpSendItem_t *r, const aMediaSource_t *s){
  char localHostName[100];
  struct hostent *localHost;
  
  r->common.version=2;
  r->common.p=0;
  r->common.count=1;
  r->common.pt=(uchar_t)202;
  r->common.length=htons(2);
  srand(time(0));
  r->sdes.src=(unsigned long)rand();
  
  r->sdes.item.type=1;
									  
  gethostname(localHostName,100);
  localHost=gethostbyname(localHostName);
  inet_ntop(AF_INET,localHost->h_addr_list[0],localHostName,100);
    
  r->sdes.item.length=strlen(localHostName);
  memcpy(r->sdes.item.data,localHostName,strlen(localHostName));
  
}

