/************************************************************
 *
 * $Id: rtp_video_listener.c,v 1.5 2000/05/03 19:25:05 htran Exp $
 *
 * Desc: This file IS the entire thread for video.
 *       RTP/RTCP socket pair is opened. Both are "listened" for activity 
 *       with the fd_sets . 
 *       
 *       RTP packets received are handled immediately.
 *       RTCP packets received are replied to on every packet received from
 *       the source. 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ************************************************************/

#include <netdb.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include "listeners.h"
#include "arguments.h"
#include "start_msock.h"
#include "uTypes.h"


extern pthread_mutex_t playingLock;

static void 
close_opened_FILE(FILE *fp){
  while( fclose(fp)< 0 );
  pthread_mutex_unlock(&playingLock);
}
static void 
close_opened_fd(int *fd){
  while( close(*fd)< 0 );
}


static void 
fill_send_item(rtcpSendItem_t *sendBuffer, const aMediaSource_t *source);
#ifdef NEED_INET_NTOP
char *
inet_ntop(int family, const char *addrptr, char *strprt, size_t len);
#endif

void *
rtp_video_listener(args_t *args){ 
  FILE *fpOut;
  int errorStatus=-1;
  int pthreadLastState;
  
  fd_set readFdSet, activeFdSet;
  
  rtcpSendItem_t sItem;
  struct sockaddr_in sendToAddress;
  socklen_t addressSize=sizeof(struct sockaddr_in);

  rtcp_t recvRTCP;
  
  rtpVideoPacket_t recvPacket;
  size_t rtpPacketSize=sizeof(recvPacket);
  int length=0;
  int sendSize=0;
  
  aMediaSource_t videoSource;
  extern pthread_mutex_t argumentsLock;
  
  int i=0;
  
  
  if(args==NULL)
	 pthread_exit(&errorStatus);
  
  /* lock argumentsLock, copy data, unlock */
  pthread_mutex_lock(&argumentsLock);
  strcpy(videoSource.address, args->videoAddress);
  videoSource.port=args->videoPort;
  videoSource.zoom=args->zoom;
  videoSource.enabled=args->playVideo;
  pthread_mutex_unlock(&argumentsLock);
  
  
  if( !videoSource.enabled )
	 pthread_exit(&errorStatus);
  else{
	 if( (videoSource.rtcpsock=
			start_msock(videoSource.address,videoSource.port+1,1,127,0))< 0 ){
		perror("rtcp_socks[video]=start_msock()");
		fprintf(stderr,"%s %d\n", videoSource.address, videoSource.port);
		exit(-1);
	 }
	 else{
		fprintf(stderr,"rtcpVideo started\n");
	 }
	 if( (videoSource.rtpsock=
			start_msock(videoSource.address,videoSource.port,1,127,0))< 0 ){
		perror("rtp_socks[video]=start_msock()");
		exit(-1);
	 } 
	 else{
		fprintf(stderr,"rtpVideo started\n");
	 }
  }
  /* if we get here we know that video will be played */
  pthread_mutex_lock(&playingLock);
  
  if(videoSource.zoom){
    if( (fpOut=popen("mtvp fd:0 -a0 -z","w"))==NULL ){
		fprintf(stderr,"%d:: ",(int)pthread_self());
		perror("popen()");
		pthread_exit(&errorStatus);
	 }
  }
  else{
    if( (fpOut=popen("mtvp fd:0 -a0","w"))==NULL ){
		fprintf(stderr,"%d:: ",(int)pthread_self());
		perror("popen()");
		pthread_exit(&errorStatus);
	 }
  }
  
  /* fill rtcpSendItem_t sItem */
  fill_send_item(&sItem,&videoSource);
  fprintf(stderr,"%s\n", sItem.sdes.item.data);
  
  /* set up destination address */
  sendToAddress.sin_family=AF_INET;
  sendToAddress.sin_addr.s_addr=inet_addr(videoSource.address);
  sendToAddress.sin_port=htons(videoSource.port+1);
  
  /* set to asynchronous cancel */
  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &pthreadLastState);

  /* set up file sets */
  FD_ZERO(&activeFdSet);
  FD_SET(videoSource.rtpsock,&activeFdSet);
  FD_SET(videoSource.rtcpsock,&activeFdSet);
  
  /* establish cleanup handlers for the open socks and FILE */
  pthread_cleanup_push((void *)close_opened_FILE,(void *)fpOut);
  pthread_cleanup_push((void *)close_opened_fd,(void *)&(videoSource.rtpsock));
  pthread_cleanup_push((void *)close_opened_fd,
							  (void *)&(videoSource.rtcpsock));
  pthread_cleanup_push((void *)stop_msock,(void *)&videoSource);
  
  /* enter main loop and don't leave */
  

  while(1){	
	 readFdSet=activeFdSet;
	 memset(&recvPacket,0,rtpPacketSize);
	 memset(recvPacket.data,0,BUFFERSIZE);   
	 sendSize=0;
	 length=0;
	 if (select (FD_SETSIZE, &readFdSet, NULL, NULL, NULL) < 0){
		perror ("select");
		exit (-1);
	 }
	 for(i=0;i<FD_SETSIZE;++i){
		if( FD_ISSET(i,&readFdSet) ){
		  if( i==videoSource.rtpsock ){   // RTP Ready
			 
			 if( (length=recvfrom(videoSource.rtpsock,
										 (rtpVideoPacket_t *)&recvPacket,
										 rtpPacketSize,0,
										 NULL,NULL)) < 0 ){
				perror("rtp:recvfrom()");
				break;
				
			 }
			 sendSize=length-sizeof(rtpHeader_t)-sizeof(rtpVideoHeader_t);
			 fwrite(recvPacket.data,sendSize,1,fpOut); 
		  }
		  else if( i==videoSource.rtcpsock ){   //RTCP Ready
				
			 memset(&recvRTCP,0,sizeof(recvRTCP));
			 if( (length=recvfrom(videoSource.rtcpsock,(rtcp_t *)&recvRTCP,
										 sizeof(recvRTCP),0,
										 NULL,NULL)) < 0 ){
				perror("rtcp:recvfrom()");
				pthread_exit(&length);
			 }
			 
			 if(recvRTCP.common.pt==200){
				if( sendto(videoSource.rtcpsock,(rtcp_t *)&sItem,
							  sizeof(rtcpSendItem_t),0,
							  (struct sockaddr *)&sendToAddress,addressSize)<0 ){
				  fprintf(stderr,"%d:: ", (int)pthread_self());
				  perror("rtcp:sendto()");
				}
				
			 }
			 
		  }
		  
		}
	 }
	 
	 
  }
  

  
  
  
  /* should never get to here.
	  the thread should just loop until it gets a cancel signal from
	  parent.
	  
  */
  return NULL;
  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 
  pthread_cleanup_pop(0); 

}

static void 
fill_send_item(rtcpSendItem_t *r, const aMediaSource_t *s){
  char localHostName[100];
  struct hostent *localHost;
  
  r->common.version=2;
  r->common.p=0;
  r->common.count=1;
  r->common.pt=(uchar_t)202;
  r->common.length=htons(2);
  srand(time(0));
  r->sdes.src=(unsigned long)rand();
  
  r->sdes.item.type=1;
									  
  gethostname(localHostName,100);
  localHost=gethostbyname(localHostName);
  inet_ntop(AF_INET,localHost->h_addr_list[0],localHostName,100);
    
  r->sdes.item.length=strlen(localHostName);
  memcpy(r->sdes.item.data,localHostName,strlen(localHostName));
  
}


#ifdef NEED_INET_NTOP
char *
inet_ntop(int family, const char *addrptr, char *strptr, size_t len){
  
  const u_char *p=(const u_char *)addrptr;
  int errno;
  if(family==AF_INET){
	 char temp[16];
	 sprintf(temp,"%d.%d.%d.%d",p[0],p[1],p[2],p[3]);  
	 if(strlen(temp)>=len){
		errno=-1;
		return (NULL);
	 }
	 strcpy(strptr,temp);
		return (strptr);
  }
  errno=-2;
  return (NULL);
  
}
#endif
